@@ -40,15 +40,14 @@
 #include "google/protobuf/struct.upb.h"
 #include "google/protobuf/timestamp.upb.h"
 #include "google/rpc/status.upb.h"
-#include "upb/def.h"
-#include "upb/text_encode.h"
-#include "upb/upb.h"
+#include "upb/base/string_view.h"
+#include "upb/reflection/def.h"
+#include "upb/text/encode.h"
 #include "upb/upb.hpp"
 
-#include <grpc/grpc.h>
-#include <grpc/impl/codegen/gpr_types.h>
 #include <grpc/status.h>
 #include <grpc/support/log.h>
+#include <grpc/support/time.h>
 
 #include "src/core/ext/xds/upb_utils.h"
 #include "src/core/ext/xds/xds_client.h"
@@ -58,40 +57,15 @@
 
 namespace grpc_core {
 
-// If gRPC is built with -DGRPC_XDS_USER_AGENT_NAME_SUFFIX="...", that string
-// will be appended to the user agent name reported to the xDS server.
-#ifdef GRPC_XDS_USER_AGENT_NAME_SUFFIX
-#define GRPC_XDS_USER_AGENT_NAME_SUFFIX_STRING \
-  " " GRPC_XDS_USER_AGENT_NAME_SUFFIX
-#else
-#define GRPC_XDS_USER_AGENT_NAME_SUFFIX_STRING ""
-#endif
-
-// If gRPC is built with -DGRPC_XDS_USER_AGENT_VERSION_SUFFIX="...", that string
-// will be appended to the user agent version reported to the xDS server.
-#ifdef GRPC_XDS_USER_AGENT_VERSION_SUFFIX
-#define GRPC_XDS_USER_AGENT_VERSION_SUFFIX_STRING \
-  " " GRPC_XDS_USER_AGENT_VERSION_SUFFIX
-#else
-#define GRPC_XDS_USER_AGENT_VERSION_SUFFIX_STRING ""
-#endif
-
 XdsApi::XdsApi(XdsClient* client, TraceFlag* tracer,
-               const XdsBootstrap::Node* node, upb::SymbolTable* symtab)
+               const XdsBootstrap::Node* node, upb::SymbolTable* symtab,
+               std::string user_agent_name, std::string user_agent_version)
     : client_(client),
       tracer_(tracer),
       node_(node),
       symtab_(symtab),
-      build_version_(absl::StrCat("gRPC C-core ", GPR_PLATFORM_STRING, " ",
-                                  grpc_version_string(),
-                                  GRPC_XDS_USER_AGENT_NAME_SUFFIX_STRING,
-                                  GRPC_XDS_USER_AGENT_VERSION_SUFFIX_STRING)),
-      user_agent_name_(absl::StrCat("gRPC C-core ", GPR_PLATFORM_STRING,
-                                    GRPC_XDS_USER_AGENT_NAME_SUFFIX_STRING)),
-      user_agent_version_(
-          absl::StrCat("C-core ", grpc_version_string(),
-                       GRPC_XDS_USER_AGENT_NAME_SUFFIX_STRING,
-                       GRPC_XDS_USER_AGENT_VERSION_SUFFIX_STRING)) {}
+      user_agent_name_(std::move(user_agent_name)),
+      user_agent_version_(std::move(user_agent_version)) {}
 
 namespace {
 
@@ -100,7 +74,6 @@ struct XdsApiContext {
   TraceFlag* tracer;
   upb_DefPool* symtab;
   upb_Arena* arena;
-  bool use_v3;
 };
 
 void PopulateMetadataValue(const XdsApiContext& context,
@@ -130,111 +103,74 @@ void PopulateMetadata(const XdsApiContext& context,
 void PopulateMetadataValue(const XdsApiContext& context,
                            google_protobuf_Value* value_pb, const Json& value) {
   switch (value.type()) {
-    case Json::Type::JSON_NULL:
+    case Json::Type::kNull:
       google_protobuf_Value_set_null_value(value_pb, 0);
       break;
-    case Json::Type::NUMBER:
+    case Json::Type::kNumber:
       google_protobuf_Value_set_number_value(
-          value_pb, strtod(value.string_value().c_str(), nullptr));
+          value_pb, strtod(value.string().c_str(), nullptr));
       break;
-    case Json::Type::STRING:
+    case Json::Type::kString:
       google_protobuf_Value_set_string_value(
-          value_pb, StdStringToUpbString(value.string_value()));
+          value_pb, StdStringToUpbString(value.string()));
       break;
-    case Json::Type::JSON_TRUE:
+    case Json::Type::kTrue:
       google_protobuf_Value_set_bool_value(value_pb, true);
       break;
-    case Json::Type::JSON_FALSE:
+    case Json::Type::kFalse:
       google_protobuf_Value_set_bool_value(value_pb, false);
       break;
-    case Json::Type::OBJECT: {
+    case Json::Type::kObject: {
       google_protobuf_Struct* struct_value =
           google_protobuf_Value_mutable_struct_value(value_pb, context.arena);
-      PopulateMetadata(context, struct_value, value.object_value());
+      PopulateMetadata(context, struct_value, value.object());
       break;
     }
-    case Json::Type::ARRAY: {
+    case Json::Type::kArray: {
       google_protobuf_ListValue* list_value =
           google_protobuf_Value_mutable_list_value(value_pb, context.arena);
-      PopulateListValue(context, list_value, value.array_value());
+      PopulateListValue(context, list_value, value.array());
       break;
     }
   }
 }
 
-// Helper functions to manually do protobuf string encoding, so that we
-// can populate the node build_version field that was removed in v3.
-std::string EncodeVarint(uint64_t val) {
-  std::string data;
-  do {
-    uint8_t byte = val & 0x7fU;
-    val >>= 7;
-    if (val) byte |= 0x80U;
-    data += byte;
-  } while (val);
-  return data;
-}
-std::string EncodeTag(uint32_t field_number, uint8_t wire_type) {
-  return EncodeVarint((field_number << 3) | wire_type);
-}
-std::string EncodeStringField(uint32_t field_number, const std::string& str) {
-  static const uint8_t kDelimitedWireType = 2;
-  return EncodeTag(field_number, kDelimitedWireType) +
-         EncodeVarint(str.size()) + str;
-}
-
-void PopulateBuildVersion(const XdsApiContext& context,
-                          envoy_config_core_v3_Node* node_msg,
-                          const std::string& build_version) {
-  std::string encoded_build_version = EncodeStringField(5, build_version);
-  // TODO(roth): This should use upb_Message_AddUnknown(), but that API is
-  // broken in the current version of upb, so we're using the internal
-  // API for now.  Change this once we upgrade to a version of upb that
-  // fixes this bug.
-  _upb_Message_AddUnknown(node_msg, encoded_build_version.data(),
-                          encoded_build_version.size(), context.arena);
-}
-
 void PopulateNode(const XdsApiContext& context, const XdsBootstrap::Node* node,
-                  const std::string& build_version,
                   const std::string& user_agent_name,
                   const std::string& user_agent_version,
                   envoy_config_core_v3_Node* node_msg) {
   if (node != nullptr) {
-    if (!node->id.empty()) {
+    if (!node->id().empty()) {
       envoy_config_core_v3_Node_set_id(node_msg,
-                                       StdStringToUpbString(node->id));
+                                       StdStringToUpbString(node->id()));
     }
-    if (!node->cluster.empty()) {
+    if (!node->cluster().empty()) {
       envoy_config_core_v3_Node_set_cluster(
-          node_msg, StdStringToUpbString(node->cluster));
+          node_msg, StdStringToUpbString(node->cluster()));
     }
-    if (!node->metadata.object_value().empty()) {
+    if (!node->metadata().empty()) {
       google_protobuf_Struct* metadata =
           envoy_config_core_v3_Node_mutable_metadata(node_msg, context.arena);
-      PopulateMetadata(context, metadata, node->metadata.object_value());
+      PopulateMetadata(context, metadata, node->metadata());
     }
-    if (!node->locality_region.empty() || !node->locality_zone.empty() ||
-        !node->locality_sub_zone.empty()) {
+    if (!node->locality_region().empty() || !node->locality_zone().empty() ||
+        !node->locality_sub_zone().empty()) {
       envoy_config_core_v3_Locality* locality =
           envoy_config_core_v3_Node_mutable_locality(node_msg, context.arena);
-      if (!node->locality_region.empty()) {
+      if (!node->locality_region().empty()) {
         envoy_config_core_v3_Locality_set_region(
-            locality, StdStringToUpbString(node->locality_region));
+            locality, StdStringToUpbString(node->locality_region()));
       }
-      if (!node->locality_zone.empty()) {
+      if (!node->locality_zone().empty()) {
         envoy_config_core_v3_Locality_set_zone(
-            locality, StdStringToUpbString(node->locality_zone));
+            locality, StdStringToUpbString(node->locality_zone()));
       }
-      if (!node->locality_sub_zone.empty()) {
+      if (!node->locality_sub_zone().empty()) {
         envoy_config_core_v3_Locality_set_sub_zone(
-            locality, StdStringToUpbString(node->locality_sub_zone));
+            locality, StdStringToUpbString(node->locality_sub_zone()));
       }
     }
   }
-  if (!context.use_v3) {
-    PopulateBuildVersion(context, node_msg, build_version);
-  }
   envoy_config_core_v3_Node_set_user_agent_name(
       node_msg, StdStringToUpbString(user_agent_name));
   envoy_config_core_v3_Node_set_user_agent_version(
@@ -271,13 +207,11 @@ std::string SerializeDiscoveryRequest(
 }  // namespace
 
 std::string XdsApi::CreateAdsRequest(
-    const XdsBootstrap::XdsServer& server, absl::string_view type_url,
-    absl::string_view version, absl::string_view nonce,
-    const std::vector<std::string>& resource_names, absl::Status status,
-    bool populate_node) {
+    absl::string_view type_url, absl::string_view version,
+    absl::string_view nonce, const std::vector<std::string>& resource_names,
+    absl::Status status, bool populate_node) {
   upb::Arena arena;
-  const XdsApiContext context = {client_, tracer_, symtab_->ptr(), arena.ptr(),
-                                 server.ShouldUseV3()};
+  const XdsApiContext context = {client_, tracer_, symtab_->ptr(), arena.ptr()};
   // Create a request.
   envoy_service_discovery_v3_DiscoveryRequest* request =
       envoy_service_discovery_v3_DiscoveryRequest_new(arena.ptr());
@@ -317,8 +251,8 @@ std::string XdsApi::CreateAdsRequest(
     envoy_config_core_v3_Node* node_msg =
         envoy_service_discovery_v3_DiscoveryRequest_mutable_node(request,
                                                                  arena.ptr());
-    PopulateNode(context, node_, build_version_, user_agent_name_,
-                 user_agent_version_, node_msg);
+    PopulateNode(context, node_, user_agent_name_, user_agent_version_,
+                 node_msg);
     envoy_config_core_v3_Node_add_client_features(
         node_msg, upb_StringView_FromString("xds.config.resource-in-sotw"),
         context.arena);
@@ -350,12 +284,10 @@ void MaybeLogDiscoveryResponse(
 
 }  // namespace
 
-absl::Status XdsApi::ParseAdsResponse(const XdsBootstrap::XdsServer& server,
-                                      absl::string_view encoded_response,
+absl::Status XdsApi::ParseAdsResponse(absl::string_view encoded_response,
                                       AdsResponseParserInterface* parser) {
   upb::Arena arena;
-  const XdsApiContext context = {client_, tracer_, symtab_->ptr(), arena.ptr(),
-                                 server.ShouldUseV3()};
+  const XdsApiContext context = {client_, tracer_, symtab_->ptr(), arena.ptr()};
   // Decode the response.
   const envoy_service_discovery_v3_DiscoveryResponse* response =
       envoy_service_discovery_v3_DiscoveryResponse_parse(
@@ -390,13 +322,13 @@ absl::Status XdsApi::ParseAdsResponse(const XdsBootstrap::XdsServer& server,
     absl::string_view serialized_resource =
         UpbStringToAbsl(google_protobuf_Any_value(resources[i]));
     // Unwrap Resource messages, if so wrapped.
-    if (type_url == "envoy.api.v2.Resource" ||
-        type_url == "envoy.service.discovery.v3.Resource") {
+    absl::string_view resource_name;
+    if (type_url == "envoy.service.discovery.v3.Resource") {
       const auto* resource_wrapper = envoy_service_discovery_v3_Resource_parse(
           serialized_resource.data(), serialized_resource.size(), arena.ptr());
       if (resource_wrapper == nullptr) {
-        return absl::InvalidArgumentError(
-            "Can't decode Resource proto wrapper");
+        parser->ResourceWrapperParsingFailed(i);
+        continue;
       }
       const auto* resource =
           envoy_service_discovery_v3_Resource_resource(resource_wrapper);
@@ -405,8 +337,11 @@ absl::Status XdsApi::ParseAdsResponse(const XdsBootstrap::XdsServer& server,
           "type.googleapis.com/");
       serialized_resource =
           UpbStringToAbsl(google_protobuf_Any_value(resource));
+      resource_name = UpbStringToAbsl(
+          envoy_service_discovery_v3_Resource_name(resource_wrapper));
     }
-    parser->ParseResource(context.arena, i, type_url, serialized_resource);
+    parser->ParseResource(context.arena, i, type_url, resource_name,
+                          serialized_resource);
   }
   return absl::OkStatus();
 }
@@ -438,11 +373,9 @@ std::string SerializeLrsRequest(
 
 }  // namespace
 
-std::string XdsApi::CreateLrsInitialRequest(
-    const XdsBootstrap::XdsServer& server) {
+std::string XdsApi::CreateLrsInitialRequest() {
   upb::Arena arena;
-  const XdsApiContext context = {client_, tracer_, symtab_->ptr(), arena.ptr(),
-                                 server.ShouldUseV3()};
+  const XdsApiContext context = {client_, tracer_, symtab_->ptr(), arena.ptr()};
   // Create a request.
   envoy_service_load_stats_v3_LoadStatsRequest* request =
       envoy_service_load_stats_v3_LoadStatsRequest_new(arena.ptr());
@@ -450,8 +383,7 @@ std::string XdsApi::CreateLrsInitialRequest(
   envoy_config_core_v3_Node* node_msg =
       envoy_service_load_stats_v3_LoadStatsRequest_mutable_node(request,
                                                                 arena.ptr());
-  PopulateNode(context, node_, build_version_, user_agent_name_,
-               user_agent_version_, node_msg);
+  PopulateNode(context, node_, user_agent_name_, user_agent_version_, node_msg);
   envoy_config_core_v3_Node_add_client_features(
       node_msg,
       upb_StringView_FromString("envoy.lrs.supports_send_all_clusters"),
@@ -513,8 +445,7 @@ void LocalityStatsPopulate(
 std::string XdsApi::CreateLrsRequest(
     ClusterLoadReportMap cluster_load_report_map) {
   upb::Arena arena;
-  const XdsApiContext context = {client_, tracer_, symtab_->ptr(), arena.ptr(),
-                                 /*use_v3=*/false};
+  const XdsApiContext context = {client_, tracer_, symtab_->ptr(), arena.ptr()};
   // Create a request.
   envoy_service_load_stats_v3_LoadStatsRequest* request =
       envoy_service_load_stats_v3_LoadStatsRequest_new(arena.ptr());
@@ -573,6 +504,24 @@ std::string XdsApi::CreateLrsRequest(
   return SerializeLrsRequest(context, request);
 }
 
+namespace {
+
+void MaybeLogLrsResponse(
+    const XdsApiContext& context,
+    const envoy_service_load_stats_v3_LoadStatsResponse* response) {
+  if (GRPC_TRACE_FLAG_ENABLED(*context.tracer) &&
+      gpr_should_log(GPR_LOG_SEVERITY_DEBUG)) {
+    const upb_MessageDef* msg_type =
+        envoy_service_load_stats_v3_LoadStatsResponse_getmsgdef(context.symtab);
+    char buf[10240];
+    upb_TextEncode(response, msg_type, nullptr, 0, buf, sizeof(buf));
+    gpr_log(GPR_DEBUG, "[xds_client %p] received LRS response: %s",
+            context.client, buf);
+  }
+}
+
+}  // namespace
+
 absl::Status XdsApi::ParseLrsResponse(absl::string_view encoded_response,
                                       bool* send_all_clusters,
                                       std::set<std::string>* cluster_names,
@@ -586,6 +535,8 @@ absl::Status XdsApi::ParseLrsResponse(absl::string_view encoded_response,
   if (decoded_response == nullptr) {
     return absl::UnavailableError("Can't decode response.");
   }
+  const XdsApiContext context = {client_, tracer_, symtab_->ptr(), arena.ptr()};
+  MaybeLogLrsResponse(context, decoded_response);
   // Check send_all_clusters.
   if (envoy_service_load_stats_v3_LoadStatsResponse_send_all_clusters(
           decoded_response)) {
@@ -632,10 +583,8 @@ std::string XdsApi::AssembleClientConfig(
   // Fill-in the node information
   auto* node = envoy_service_status_v3_ClientConfig_mutable_node(client_config,
                                                                  arena.ptr());
-  const XdsApiContext context = {client_, tracer_, symtab_->ptr(), arena.ptr(),
-                                 /*use_v3=*/true};
-  PopulateNode(context, node_, build_version_, user_agent_name_,
-               user_agent_version_, node);
+  const XdsApiContext context = {client_, tracer_, symtab_->ptr(), arena.ptr()};
+  PopulateNode(context, node_, user_agent_name_, user_agent_version_, node);
   // Dump each resource.
   std::vector<std::string> type_url_storage;
   for (const auto& p : resource_type_metadata_map) {
